
#include "TNadekoCanvasManager.hh"

#include "TROOT.h"
#include "TCanvas.h"
#include "TPad.h"
#include "TVirtualPad.h"
#include "TList.h"

#include <iostream>

void TNadekoCanvasManager::SetLogx(Int_t value)
{
  gPad->SetLogx(value);
}

void TNadekoCanvasManager::SetLogy(Int_t value)
{
  gPad->SetLogy(value);
}

void TNadekoCanvasManager::SetLogz(Int_t value)
{
  gPad->SetLogz(value);
}

TCanvas* TNadekoCanvasManager::GetCanvas()
{
//  TList* list = static_cast<TList*>(gROOT->GetListOfCanvases());
//  TObject* obj = list->Last();
//  if(obj && obj->InheritsFrom(TCanvas::Class())){
//    return static_cast<TCanvas*>(obj);
//  }else{
//    return 0;
//  }
  if(!gPad) return 0;
  return gPad->GetCanvas();
}

TCanvas* TNadekoCanvasManager::GetCanvas(Int_t idx)
{
  TList* list = static_cast<TList*>(gROOT->GetListOfCanvases());

  TObject* obj = list->At(idx);
  if(obj && obj->InheritsFrom(TCanvas::Class())){
    return static_cast<TCanvas*>(obj);
  }else{
    return 0;
  }
}

Int_t TNadekoCanvasManager::GetSubpadNumber()
{
  TCanvas* canvas = GetCanvas();
  if(!canvas) return 0;  
  TList* list = canvas->GetListOfPrimitives();
  if(!list) return 0;

  Int_t subpadnumber = 0;
  TObject *obj;
  TIter next(list);
  while((obj = next())){
    if(obj->InheritsFrom(TPad::Class())){
      ++subpadnumber;
    }
  }

  return subpadnumber;
}

TObject* TNadekoCanvasManager::GetObject(TClass* classpt)
{
  if(!gPad) return 0;

  TList* list = gPad->GetListOfPrimitives();
  TObjLink* objlink = list->FirstLink();
  while(objlink){
    TObject* obj2 = objlink->GetObject();
    if(obj2->InheritsFrom(classpt)) return obj2;
    objlink = objlink->Next();
  }

  return 0;
}

void TNadekoCanvasManager::SetWindowSize(UInt_t ww, UInt_t wh)
{
  if(ww < 100 || wh < 100) return;

  TCanvas* canvas = GetCanvas();
  if(!canvas) return;

  canvas->SetWindowSize(ww, wh);
}

void TNadekoCanvasManager::Divide(Int_t nx, Int_t ny, Float_t xmargin, Float_t ymargin, Int_t color)
{
  TCanvas* canvas = GetCanvas();
  if(!canvas) return;

  if(nx < 1 || ny < 1){
    std::cout << "nx and ny must be positive number" << std::endl;
    return;
  }

  canvas->Clear();
  canvas->Divide(nx, ny, xmargin, ymargin, color);
  canvas->cd(1);
}

TVirtualPad* TNadekoCanvasManager::cd(Int_t subpadnumber)
{
  TCanvas* canvas = GetCanvas();
  if(!canvas) return 0;  

  Int_t subpad = GetSubpadNumber();
  if(subpad == 0){
    canvas->cd(0);
    return 0;
  }else{
    return canvas->cd((subpadnumber-1)%subpad+1);
  }
}

TVirtualPad* TNadekoCanvasManager::cdNext()
{
  TCanvas* canvas = GetCanvas();
  if(!canvas) return 0;
  TList* list = canvas->GetListOfPrimitives();
  if(!list) return 0;
  TObjLink* objlink = list->FirstLink();
  while(objlink){
    TObject* obj = objlink->GetObject();
    if(obj && obj->InheritsFrom(TPad::Class())){
      TPad* pad = static_cast<TPad*>(obj);
      if(pad->IsEqual(gPad)){
	objlink = objlink->Next();      
	if(!objlink) objlink = list->FirstLink();
	pad = static_cast<TPad*>(objlink->GetObject());
	return pad->cd();
      }
    }
    objlink = objlink->Next();
  }  

  return 0;
}

TVirtualPad* TNadekoCanvasManager::cdPrev()
{
  TCanvas* canvas = GetCanvas();
  if(!canvas) return 0;
  TList* list = canvas->GetListOfPrimitives();
  if(!list) return 0;
  TObjLink* objlink = list->LastLink();
  while(objlink){
    TObject* obj = objlink->GetObject();
    if(obj && obj->InheritsFrom(TPad::Class())){
      TPad* pad = static_cast<TPad*>(obj);
      if(pad->IsEqual(gPad)){
	objlink = objlink->Prev();      
	if(!objlink) objlink = list->LastLink();
	pad = static_cast<TPad*>(objlink->GetObject());
	return pad->cd();
      }
    }
    objlink = objlink->Prev();
  }  

  return 0;
}

void TNadekoCanvasManager::SaveAs(char* filename, Option_t* option)
{
  TCanvas* canvas = GetCanvas();
  if(!canvas) return;

  canvas->SaveAs(filename, option);
}

